import {Component, EventEmitter, ViewChild} from '@angular/core';
import {MatAccordion, MatExpansionModule, MatExpansionPanel} from "@angular/material/expansion";
import {MatChipListbox, MatChipsModule} from "@angular/material/chips";
import {catchError} from "rxjs";
import {FormControl, ReactiveFormsModule} from "@angular/forms";
import {LoadingIndicatorComponent} from "@component/common/loading-indicator/loading-indicator.component";
import {CampusBasicInfoResponse} from "@api/v1/campusBasicInfoResponse";
import {LectureHallResponse} from "@api/v1/lectureHallResponse";
import {CampusService} from "@api/v1/campus.service";
import {LectureHallService} from "@api/v1/lectureHall.service";
import {IScheduleTab} from "@component/schedule/tabs/ischedule-tab";
import {TabSelect, TabStorageService} from "@service/tab-storage.service";

enum Enclosure {
  campus, lecture
}

@Component({
  selector: 'app-lecture-hall',
  standalone: true,
  imports: [
    MatChipsModule,
    MatExpansionModule,
    ReactiveFormsModule,
    MatAccordion,
    LoadingIndicatorComponent
  ],
  templateUrl: './lecture-hall.component.html',
  styleUrl: './lecture-hall.component.css',
  providers: [CampusService, LectureHallService]
})
export class LectureHallComponent implements IScheduleTab {
  protected campusId: number | null = null;
  protected formLectureHalls: FormControl = new FormControl();

  protected campuses: CampusBasicInfoResponse[] | null = null;
  protected lectureHallsFiltered: LectureHallResponse[] | null = null;

  @ViewChild('lecturePanel') lecturePanel!: MatExpansionPanel;
  @ViewChild('lectureIndicator') lectureIndicator!: LoadingIndicatorComponent;
  @ViewChild('campusIndicator') campusIndicator!: LoadingIndicatorComponent;

  @ViewChild('campusChip') campusChip!: MatChipListbox;
  @ViewChild('lectureChip') lectureChip!: MatChipListbox;

  private lectureHalls: LectureHallResponse[] | null = null;

  public eventResult = new EventEmitter<number>();
  public selectChangeEvent = new EventEmitter<TabSelect[]>();

  constructor(private campusApi: CampusService, private lectureHallApi: LectureHallService) {
  }

  private getSelectedTabs(): TabSelect[] {
    const campus = this.campusChip.value;
    const lecture = this.lectureChip.value;

    const result: TabSelect[] = [];

    if (campus)
      result.push(new TabSelect(campus, this.campuses!.find(x => x.id === campus)?.codeName ?? ''));

    if (lecture)
      result.push(new TabSelect(lecture, this.lectureHallsFiltered!.find(x => x.id === lecture)?.name ?? ''));

    return result;
  }

  getEnclosureList(): string[] {
    return Object.keys(Enclosure).filter((item) => {
      return isNaN(Number(item));
    });
  }

  protected loadCampuses() {
    this.campusApi.getCampus()
      .pipe(catchError(error => {
        this.campusIndicator.loading = false;
        throw error;
      }))
      .subscribe(data => {
        this.campuses = data;

        let selected = TabStorageService.selected?.selected[this.getEnclosureList()[Enclosure.campus]];
        if (selected) {
          let selectedCampus = data.find(x => x.id === selected.index);

          if (selectedCampus === undefined || selectedCampus.codeName !== selected.name)
            selectedCampus = data.find(x => x.codeName === selected.name);

          if (selectedCampus !== undefined) {
            TabStorageService.trySelectChip(selectedCampus.id, this.campusChip);
            this.onCampusSelected(selectedCampus.id, true);
          }
        }
      });
  }

  private filteringLectureHalls() {
    this.lectureHallsFiltered = this.lectureHalls?.filter(x => x.campusId === this.campusId) ?? null;
  }

  protected onCampusSelected(index: number, loadMode: boolean = false) {
    this.formLectureHalls.reset();
    this.lectureChip.value = undefined;

    if (loadMode)
      this.campusChip.value = index;
    else
      this.selectChangeEvent.emit(this.getSelectedTabs());

    if (index === undefined) {
      this.campusId = null;
      this.lectureHalls = [];
      return;
    }

    this.campusId = index;
    this.lecturePanel.open();

    if (this.lectureHalls === null)
      this.loadLectureHalls();
    else
      this.filteringLectureHalls();
  }

  protected loadLectureHalls() {
    this.lectureHallApi.getLectureHalls()
      .pipe(catchError(error => {
        this.lectureIndicator.loading = false;
        throw error;
      }))
      .subscribe(data => {
        this.lectureHalls = data;
        this.filteringLectureHalls();

        let selected = TabStorageService.selected?.selected[this.getEnclosureList()[Enclosure.lecture]];
        if (selected) {
          let selectedLecture = data.find(x => x.id === selected.index);

          if (selectedLecture === undefined || selectedLecture.name !== selected.name)
            selectedLecture = data.find(x => x.name === selected.name);

          if (selectedLecture !== undefined) {
            TabStorageService.trySelectChip(selectedLecture.id, this.lectureChip);
            this.onLectureHallSelected(selectedLecture.id, true);
          }
        }
      });
  }

  protected onLectureHallSelected(index: number, loadMode: boolean = false) {
    if (index === undefined)
      return;

    if (loadMode)
      this.lectureChip.value = index;
    else
      this.selectChangeEvent.emit(this.getSelectedTabs());

    this.lecturePanel.close();
    this.eventResult.emit(index);
  }

  public load() {
    if (this.campuses === null)
      this.loadCampuses();
  }
}
